/*****************************************************************************
* PELICAN crossing example
*
* Copyright (c) 2006 Miro Samek. All rights reserved.
*
* Email: miro@quantum-leaps.com
* Web:   www.quantum-leaps.com
*****************************************************************************/
#include "qpn_port.h"
#include "qassert.h"
#include "bsp.h"

#define LED_red(brightness_) (PCA0CPH0 = 255 - (brightness_))
#define LED_grn(brightness_) (PCA0CPH1 = 255 - (brightness_))

/*..........................................................................*/
#pragma vector=0x2B
__interrupt void timer2_ISR(void) {
   TMR2CN &= ~(1 << 7);          /* clear the Timer2 interrupt-pending flag */
   QF_tick();                    /* handle all armed time events in QF-nano */
}
/*..........................................................................*/
void BSP_init(void) {
    OSCICN = 0x07;      /* Set SYSCLK as Internal Osc 24.5 MHz (full speed) */
    RSTSRC = 0x04;                         /* Enable missing clock detector */

    PCA0CN = 0x40;                       /* Enable PCA Counter/Timer (PCA0) */
    PCA0MD = 0x00;  /* Set PCA timebase = SYSCLK / 12, disable watchdog WDT */

                                     /* Duty cycle = (256 - PCA0CPHn) / 256 */
    PCA0CPH0  = 0x00;      /* Config Mod0 PWM for a 100% initial duty cycle */
    PCA0CPH1  = 0x00;      /* Config Mod1 PWM for a 100% initial duty cycle */

    PCA0CPM0  = 0x42;                           /* Enable 8-bit PWM on CEX0 */
    PCA0CPM1  = 0x42;                           /* Enable 8-bit PWM on CEX1 */

    /* Configure the Crossbar and GPIO ports.
    * P0.0  -  Skipped
    * P0.1  -  Skipped
    * P0.2  -  CEX0 (PCA),  Push-Pull,  Digital  (Red LED)
    * P0.3  -  CEX1 (PCA),  Push-Pull,  Digital  (Green LED)
    * P0.4  -  Unassigned
    * P0.5  -  Unassigned
    * P0.6  -  Unassigned
    * P0.7  -  Unassigned
    */
    P0MDOUT = 0x0C;              /* Config P0.2 & P0.3 as push-pull outputs */
    XBR0    = 0x03;                                     /* Skip P0.0 & P0.1 */
    XBR1    = 0x80;             /* Enable PCA Capture/Compare Outputs 0 & 1 */
    XBR2    = 0x40;                                      /* Enable Crossbar */

    IP = ~0;    /* set all interrupts to the high priority (no int nesting) */

    IE_bit.ET2 = 0;                            /* disable Timer2 interrupts */
    TMR2CN  = 0x00;                    /* Stop Timer2; Clear TF2, SYSCLK/12 */
    CKCON  &= ~0x60;                      /* Timer2 clocked based on T2XCLK */

                                                            /* reload value */
    TMR2RLL  = (uint8_t)(0xFFFF - (BSP_MCK/12)/BSP_TICKS_PER_SEC);
    TMR2RLH  = (0xFFFF - (BSP_MCK/12)/BSP_TICKS_PER_SEC) >> 8;

    TMR2L    = TMR2RLL;                /* Initialize Timer2 to reload value */
    TMR2H    = TMR2RLH;                /* Initialize Timer2 to reload value */
}
/*..........................................................................*/
void QF_start(void) {
    IE_bit.ET2 = 1;                             /* Enable Timer2 interrupts */
    TMR2CN  = (1 << 2);                                     /* Start Timer2 */
}
/*..........................................................................*/
void QF_onIdle(void) {
    PCON |= 0x01;                                       /* set the IDLE bit */
    PCON = PCON;             /* ... followed by a 3-cycle dummy instruction */
}
/*..........................................................................*/
void Q_assert_handler(char const Q_ROM *file, int line) {
    file = file;                                  /* avoid compiler warning */
    line = line;                                  /* avoid compiler warning */
    QF_INT_LOCK();                  /* make sure that interrupts are locked */
    LED_red(255);
    LED_grn(0);
    for (;;) {
    }
}
/*..........................................................................*/
void BSP_signalCars(enum BSP_CarsSignal sig) {
    switch (sig) {
        case CARS_OFF:                        /* intentionally fall-through */
        case CARS_RED:
            LED_grn(0);
            break;
        case CARS_YELLOW:
            LED_grn(25);
            break;
        case CARS_GREEN:
            LED_grn(255);
            break;
    }
}
/*..........................................................................*/
void BSP_signalPeds(enum BSP_PedsSignal sig) {
    switch (sig) {
        case PEDS_DONT_WALK:
            LED_red(255);
            break;
        case PEDS_BLANK:
            LED_red(25);
            break;
        case PEDS_WALK:
            LED_red(0);
            break;
    }
}

