/*****************************************************************************
* PELICAN crossing example
*
* Copyright (c) 2006 Miro Samek. All rights reserved.
*
* Email: miro@quantum-leaps.com
* Web:   www.quantum-leaps.com
*****************************************************************************/
#include "qpn_port.h"
#include "qassert.h"
#include "bsp.h"
#include "pelican.h"

/*--------------------------------------------------------------------------*/
static QSTATE Ped_initial(Ped *me);
static QSTATE Ped_on(Ped *me);
static QSTATE Ped_wait(Ped *me);
static QSTATE Ped_off(Ped *me);

/*--------------------------------------------------------------------------*/
enum PedTimeouts {                             /* various timeouts in ticks */
    ON_TOUT   = BSP_TICKS_PER_SEC * 4,    /* wait time after posting ON_SIG */
    WAIT_TOUT = BSP_TICKS_PER_SEC * 3,  /* wait between posting PED_WAITING */
    OFF_TOUT  = BSP_TICKS_PER_SEC * 8    /* wait time after posting OFF_SIG */
};

/*..........................................................................*/
void Ped_init(Ped *me, uint8_t retry) {
    me->retry__ = retry;                           /* the number of retries */
    QHsm_init((QHsm *)me, &Ped_initial); /* take the top-most initial tran. */
}

/* HSM definition ----------------------------------------------------------*/
QSTATE Ped_initial(Ped *me) {
    switch (Q_SIG(me)) {
        case Q_TOP_INIT_SIG: {
            Q_TRAN(&Ped_on);
            return (QSTATE)0;
        }
    }
    return (QSTATE)&QHsm_top;
}
/*..........................................................................*/
QSTATE Ped_on(Ped *me) {
    switch (Q_SIG(me)) {
        case Q_ENTRY_SIG: {
            QActive_arm((QActive *)me, ON_TOUT);
            return (QSTATE)0;
        }
        case Q_TIMEOUT_SIG: {
            me->retryCtr__ = me->retry__;
            Q_TRAN(&Ped_wait);
            return (QSTATE)0;
        }
    }
    return (QSTATE)&QHsm_top;
}
/*..........................................................................*/
QSTATE Ped_wait(Ped *me) {
    switch (Q_SIG(me)) {
        case Q_ENTRY_SIG: {
            QActive_arm((QActive *)me, WAIT_TOUT);
            QF_post(PELICAN_PRIO, PEDS_WAITING_SIG, 0);
            return (QSTATE)0;
        }
        case Q_TIMEOUT_SIG: {
            if ((me->retryCtr__--) != 0) {
                Q_TRAN(&Ped_wait);                    /* transition-to-self */
            }
            else {
                Q_TRAN(&Ped_off);
            }
            return (QSTATE)0;
        }
    }
    return (QSTATE)&QHsm_top;
}
/*..........................................................................*/
QSTATE Ped_off(Ped *me) {
    switch (Q_SIG(me)) {
        case Q_ENTRY_SIG: {
            QActive_arm((QActive *)me, OFF_TOUT);
            QF_post(PELICAN_PRIO, OFF_SIG, 0);
            return (QSTATE)0;
        }
        case Q_EXIT_SIG: {
            QF_post(PELICAN_PRIO, ON_SIG, 0);
            return (QSTATE)0;
        }
        case Q_TIMEOUT_SIG: {
            Q_TRAN(&Ped_on);
            return (QSTATE)0;
        }
    }
    return (QSTATE)&QHsm_top;
}
