/*****************************************************************************
* Time Bomb example (FSM)
*
* Copyright (c) 2006 Miro Samek. All rights reserved.
*
* Email: miro@quantum-leaps.com
* Web:   www.quantum-leaps.com
*****************************************************************************/
#include "qpn_port.h"
#include "qassert.h"
#include "bsp.h"
#include "bomb.h"

/*--------------------------------------------------------------------------*/
void Bomb_off(Bomb *me);
void Bomb_timing(Bomb *me);
void Bomb_blast(Bomb *me);

/*..........................................................................*/
void Bomb_init(Bomb *me, uint8_t timeout) {
    me->timeout__ = timeout * 2;                         /* set the timeout */
    QFsm_init((QFsm *)me, (QState)&Bomb_off);  /* trigger the initial tran. */
}
/*..........................................................................*/
void Bomb_off(Bomb *me) {
    switch (Q_SIG(me)) {
        case Q_ENTRY_SIG: {
            LED_red(0);
            break;
        }
        case ARM_SIG: {
            Q_TRAN(&Bomb_timing);
            break;
        }
    }
}
/*..........................................................................*/
void Bomb_timing(Bomb *me) {
    switch (Q_SIG(me)) {
        case Q_ENTRY_SIG: {
            me->ctr__ = me->timeout__;
            QActive_arm((QActive *)me, BSP_TICKS_PER_SEC/2);
            break;
        }
        case Q_TIMEOUT_SIG: {
            QActive_arm((QActive *)me, BSP_TICKS_PER_SEC/2);
            if (me->ctr__ > (uint8_t)0) {
                if ((me->ctr__ & 0x01) == 0) {
                    LED_red(0);
                }
                else {
                    LED_red(24);
                }
                --me->ctr__;
            }
            else {                                       /* timeout expired */
                Q_TRAN(&Bomb_blast);
            }
            break;
        }
    }
}
/*..........................................................................*/
void Bomb_blast(Bomb *me) {
    switch (Q_SIG(me)) {
        case Q_ENTRY_SIG: {
            LED_red(255);
            break;
        }
        case ARM_SIG: {
            Q_TRAN(&Bomb_off);
            break;
        }
    }
}
