/*****************************************************************************
* PELICAN crossing example
*
* Copyright (c) 2006 Miro Samek. All rights reserved.
*
* Email: miro@quantum-leaps.com
* Web:   www.quantum-leaps.com
*****************************************************************************/
#include "qpn_port.h"
#include "bsp.h"

#include <conio.h>                                           /* for kbhit() */
#include <stdlib.h>                                          /* for _exit() */

/* Local-scope objects -----------------------------------------------------*/
static void interrupt (*l_dosTickISR)();
static void interrupt (*l_dosSpareISR)();

#define TICK_VECTOR     0x08
#define SPARE_VECTOR    0x81

static uint32_t l_tickCtr;

/* Direct Video Access .....................................................*/
enum VideoColor {
    /* foreground */
    VIDEO_FGND_BLACK        = 0x00,
    VIDEO_FGND_BLUE         = 0x01,
    VIDEO_FGND_GREEN        = 0x02,
    VIDEO_FGND_CYAN         = 0x03,
    VIDEO_FGND_RED          = 0x04,
    VIDEO_FGND_PURPLE       = 0x05,
    VIDEO_FGND_BROWN        = 0x06,
    VIDEO_FGND_LIGHT_GRAY   = 0x07,
    VIDEO_FGND_DARK_GRAY    = 0x08,
    VIDEO_FGND_LIGHT_BLUE   = 0x09,
    VIDEO_FGND_LIGHT_GREEN  = 0x0A,
    VIDEO_FGND_LIGHT_CYAN   = 0x0B,
    VIDEO_FGND_LIGHT_RED    = 0x0C,
    VIDEO_FGND_LIGHT_PURPLE = 0x0D,
    VIDEO_FGND_YELLOW       = 0x0E,
    VIDEO_FGND_WHITE        = 0x0F,
    /* background */
    VIDEO_BGND_BLACK        = 0x00,
    VIDEO_BGND_BLUE         = 0x10,
    VIDEO_BGND_GREEN        = 0x20,
    VIDEO_BGND_CYAN         = 0x30,
    VIDEO_BGND_RED          = 0x40,
    VIDEO_BGND_PURPLE       = 0x50,
    VIDEO_BGND_BROWN        = 0x60,
    VIDEO_BGND_LIGHT_GRAY   = 0x70,

    VIDEO_BGND_BLINK        = 0x80
};
/*..........................................................................*/
static void Video_clearScreen(uint8_t bgColor);
static void Video_clearRect(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2,
                     uint8_t bgColor);
static void Video_printChAt(uint8_t x, uint8_t y, uint8_t color, char ch);
static void Video_printStrAt(uint8_t x, uint8_t y, uint8_t color,
                      char const *str);
static void Video_printNumAt(uint8_t x, uint8_t y, uint8_t color,
                      uint32_t num);

/*..........................................................................*/
static void interrupt tickISR() {
    QF_tick();
    Video_printNumAt( 9, 19, VIDEO_FGND_YELLOW, ++l_tickCtr);
    geninterrupt(SPARE_VECTOR);     /* invoke the DOS timer ISR, see NOTE01 */
}
/*..........................................................................*/
void BSP_init(void) {
    Video_clearScreen(VIDEO_BGND_BLACK);
    Video_clearRect(11,  7, 16, 16, VIDEO_BGND_LIGHT_GRAY);
    Video_clearRect(16,  5, 73, 18, VIDEO_BGND_BLUE);
    Video_clearRect(66,  6, 69,  9, VIDEO_BGND_BLACK);
    Video_clearRect(66, 14, 69, 17, VIDEO_BGND_BLACK);
    Video_clearRect( 0, 23, 80, 24, VIDEO_BGND_LIGHT_GRAY);

    Video_printStrAt(10,  5, VIDEO_FGND_WHITE,
          "      ͻ");
    Video_printStrAt(10,  6, VIDEO_FGND_WHITE,
          "͹                           SILICON LABS         Ŀ    ");
    Video_printStrAt(10,  7, VIDEO_FGND_WHITE,
          "                                              ĴPO.3");
    Video_printStrAt(10,  8, VIDEO_FGND_WHITE,
          "                                                        ");
    Video_printStrAt(10,  9, VIDEO_FGND_WHITE,
          "                                                    ");
    Video_printStrAt(10, 10, VIDEO_FGND_WHITE,
          "        USB                             ");
    Video_printStrAt(10, 11, VIDEO_FGND_WHITE,
        "      \"F300\"    ");
    Video_printStrAt(10, 12, VIDEO_FGND_WHITE,
          "        DEBUG                           ");
    Video_printStrAt(10, 13, VIDEO_FGND_WHITE,
          "                                                    ");
    Video_printStrAt(10, 14, VIDEO_FGND_WHITE,
          "                                                    Ŀ    ");
    Video_printStrAt(10, 15, VIDEO_FGND_WHITE,
          "                                              ĴPO.2");
    Video_printStrAt(10, 16, VIDEO_FGND_WHITE,
          "͹                           TOOLSTICK                ");
    Video_printStrAt(10, 17, VIDEO_FGND_WHITE,
          "      ͼ");

    Video_printStrAt(2, 19, VIDEO_FGND_WHITE, "Tick:");

    Video_printStrAt(4, 23, VIDEO_FGND_BLUE,
         "* Copyright (c) 2006 Quantum Leaps, LLC "
         "* www.quantum-leaps.com *");
    Video_printStrAt(28, 24, VIDEO_FGND_LIGHT_RED,
         "<< Press Esc to quit >>");
}
/*..........................................................................*/
void QF_start(void) {
                                      /* save the origingal DOS vectors ... */
    l_dosTickISR  = getvect(TICK_VECTOR);
    l_dosSpareISR = getvect(SPARE_VECTOR);

    QF_INT_LOCK();                                   /* lock the interrupts */
    setvect(TICK_VECTOR, &tickISR);
    setvect(SPARE_VECTOR, l_dosTickISR);     /* install original DOS vector */
    QF_INT_UNLOCK();                               /* unlock the interrupts */
}
/*..........................................................................*/
void QF_onIdle(void) {
    if (kbhit()) {
        if (getch() == '\33') {                         /* ESC key pressed? */
            QF_exit();                                  /* exit back to DOS */
        }
    }
}
/*..........................................................................*/
void QF_exit(void) {
    QF_INT_LOCK();                                   /* lock the interrupts */
    setvect(TICK_VECTOR, l_dosTickISR);  /* restore the original DOS vector */
    setvect(SPARE_VECTOR, l_dosSpareISR);/* restore the original DOS vector */
    QF_INT_UNLOCK();                               /* unlock the interrupts */
    _exit(0);                                                /* exit to DOS */
}
/*..........................................................................*/
void BSP_signalCars(enum BSP_CarsSignal sig) {
    switch (sig) {
        case CARS_OFF:                        /* intentionally fall-through */
        case CARS_RED:
            Video_printChAt(67,  7, VIDEO_FGND_LIGHT_GREEN, ' ');
            break;
        case CARS_YELLOW:
            Video_printChAt(67,  7, VIDEO_FGND_LIGHT_GREEN, '');
            break;
        case CARS_GREEN:
            Video_printChAt(67,  7, VIDEO_FGND_LIGHT_GREEN, '');
            break;
    }
}
/*..........................................................................*/
void BSP_signalPeds(enum BSP_PedsSignal sig) {
    switch (sig) {
        case PEDS_DONT_WALK:
            Video_printChAt(67, 15, VIDEO_FGND_LIGHT_RED, '');
            break;
        case PEDS_BLANK:
            Video_printChAt(67, 15, VIDEO_FGND_LIGHT_RED, '');
            break;
        case PEDS_WALK:
            Video_printChAt(67, 15, VIDEO_FGND_LIGHT_RED, ' ');
            break;
    }
}
/*--------------------------------------------------------------------------*/
void Q_assert_handler(char const *file, int line) {
    Video_clearRect(0, 24, 80, 25, VIDEO_BGND_RED);
    Video_printStrAt( 0, 24, VIDEO_FGND_WHITE, "ASSERTION FAILED in file:");
    Video_printStrAt(26, 24, VIDEO_FGND_YELLOW, file);
    Video_printStrAt(57, 24, VIDEO_FGND_WHITE, "line:");
    Video_printNumAt(62, 24, VIDEO_FGND_YELLOW, line);

    QF_exit();                                       /* exit and cleanup QF */
}
/*--------------------------------------------------------------------------*/
void Video_clearScreen(uint8_t bgColor) {
    clrscr();
    Video_clearRect(0,  0, 80, 25, bgColor);
}
/*..........................................................................*/
void Video_clearRect(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2,
                     uint8_t bgColor)
{
    for ( ; y1 < y2; ++y1) {
        uint8_t x;
        uint8_t far *pscr = (uint8_t far *)MK_FP(0xB800,
                                           (uint16_t)(((y1 * 80) + x1) * 2));
        for (x = x1; x < x2; ++x) {
            pscr[0] = ' ';                    /* Put space in the video RAM */
            pscr[1] = bgColor;      /* Put video attribute in the video RAM */
            pscr += 2;
        }
    }
}
/*..........................................................................*/
void Video_printChAt(uint8_t x, uint8_t y, uint8_t color, char ch) {
                               /* calculate position on the video RAM (VGA) */
    uint8_t far *pscr = (uint8_t far *)MK_FP(0xB800,
                                             (uint16_t)(((y * 80) + x) * 2));
    pscr[0] = ch;
    pscr[1] = color;
}
/*..........................................................................*/
void Video_printStrAt(uint8_t x, uint8_t y, uint8_t color,
                      char const *str)
{
                               /* calculate position on the video RAM (VGA) */
    uint8_t far *pscr = (uint8_t far *)MK_FP(0xB800,
                                             (uint16_t)(((y * 80) + x) * 2));
    while (*str != (uint8_t)0) {
        pscr[0] = *str++;                     /* Put character in video RAM */
        pscr[1] |= color;               /* Put video attribute in video RAM */
        pscr += 2;
    }
}
/*..........................................................................*/
void Video_printNumAt(uint8_t x, uint8_t y, uint8_t color, uint32_t num) {
    static char buf[4];
    buf[3] = '\0';
    buf[2] = (char)('0' + num % 10);
    num /= 10;
    buf[1] = (char)('0' + num % 10);
    num /= 10;
    buf[0] = (char)('0' + num % 10);
    if (buf[0] == '0') {
        buf[0] = ' ';
    }
    Video_printStrAt(x, y, color, buf);
}
/*****************************************************************************
*
* NOTE01:
* The sofware interrupt generated at the end of the tick ISR invokes the
* original DOS tick ISR handler, which was installed at the spare vector
* 0x81. The DOS tick ISR handler generates the EOI (End-Of-Interrupt)
* to the master 8259A PIC.
*/
